/*
 * Copyright (c) 2013 F. Mannhardt (f.mannhardt@tue.nl)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package org.processmining.xeslite.external;

import java.util.concurrent.atomic.AtomicLong;

/**
 * Generates unique, sequential IDs
 * 
 * @author F. Mannhardt
 * 
 */
final class IdFactorySeq implements IdFactory {

	private final int intervalShift;
	private AtomicLong counter;

	/**
	 * Creates a new ID factory.
	 */
	IdFactorySeq(int intervalShift, Long initialValue) {
		this.intervalShift = intervalShift;
		if (initialValue != null) {
			this.counter = new AtomicLong(initialValue >> intervalShift);
		} else {
			this.counter = new AtomicLong();
		}
	}

	/**
	 * Creates a new, unique ID.
	 * 
	 * @return Unique ID.
	 */
	public long nextId() {
		return counter.incrementAndGet() << intervalShift;
	}
	
	public int getIntervalCapacity() {
		return 1 << intervalShift;
	}
	
	public int getIdShift() {
		return intervalShift;
	}
	
}